#if defined(__MINGW64__) || !defined(_WIN32)
#include <getopt.h>
#include <unistd.h>
#else
#include "src/utils/getopt_windows.h"
#endif

#include <thread>

#include "CallbackProcessor.h"
#include "sdk_common/inno_lidar_api.h"
#include "sdk_common/inno_lidar_other_api.h"
#include "sdk_common/inno_lidar_packet_utils.h"
#include "ara/exec/execution_client.h"
#include "ara/log/logging.h"
#include "LidarOption.h"
#include "FileOption.h"

#if !defined(__MINGW64__) && defined(_WIN32)
#include <io.h>
#include <stdarg.h>
#include <stdio.h>
#endif

void ThreadAct(InnoLidar &MyLidar)
{
	MyLidar.OnInit();
	MyLidar.RunThread();
}

int main(int argc, char **argv)
{
	ara::log::InitLogging("DEAS", "CM_DDS_EVENT_ASYNC_SERVER_SAMPLE",
			ara::log::LogLevel::kVerbose,
			(ara::log::LogMode::kConsole | ara::log::LogMode::kRemote));

	ara::exec::ExecutionClient execClient;
	execClient.ReportExecutionState(ara::exec::ExecutionState::kRunning);

	ara::log::Logger &mainLog
	{ ara::log::CreateLogger("main", "cm sample context",
			ara::log::LogLevel::kVerbose) };

	mainLog.LogInfo() << "Let's produce some camera data...";

	inno_log_info("inno_api_version %s", inno_api_version());
	inno_log_info("inno_api_build_tag %s", inno_api_build_tag());
	inno_log_info("inno_api_build_time %s", inno_api_build_time());

	// Set log printing level
	inno_lidar_set_log_level(INNO_LOG_LEVEL_FATAL);

	// Parse command parameters
	FileOption file_option;
	LidarOption lidar_option;

	InnoLidar InnoLidarObject(mainLog);

	// Create processing class
	CallbackProcessor processor(file_option, InnoLidarObject);

	int handle;
	if (file_option.use_pcap)
	{
		InputParam param;
		param.pcap_param.source_type = SOURCE_PCAP;
		strncpy(param.pcap_param.filename, file_option.input_filename.c_str(),
				file_option.input_filename.length() + 1);
		param.pcap_param.rewind = 0;
		param.pcap_param.data_port = file_option.lidar_udp_port;
		param.pcap_param.message_port = file_option.lidar_udp_port;
		param.pcap_param.status_port = file_option.lidar_udp_port;
		strncpy(param.pcap_param.lidar_ip, file_option.lidar_ip.c_str(),
				file_option.lidar_ip.length() + 1);
		handle = inno_lidar_open_ctx("pcap", &param);
	}
	else
	{
		handle = inno_lidar_open_live(
				"live", // name of lidar instance
				lidar_option.lidar_ip.c_str(), lidar_option.lidar_port,
				INNO_LIDAR_PROTOCOL_PCS_UDP, lidar_option.lidar_udp_port);
	}

	inno_log_verify(handle > 0, "cannot open lidar");

	int ret =
			inno_lidar_set_callbacks(handle,

					[](const int lidar_handle, void *ctx,
							const uint32_t from_remote,
							const enum InnoMessageLevel level,
							const enum InnoMessageCode code,
							const char *error_message)
							{
								return reinterpret_cast<CallbackProcessor *>(ctx)->process_message(level, code, error_message);
							},

					[](const int lidar_handle, void *ctx,
							const InnoDataPacket *pkt) -> int
							{
								inno_log_verify(pkt, "pkt");
								return reinterpret_cast<CallbackProcessor *>(ctx)->process_data(*pkt);
							},

					[](const int lidar_handle, void *ctx,
							const InnoStatusPacket *pkt) -> int
							{
								inno_log_verify(pkt, "pkt");
								return reinterpret_cast<CallbackProcessor *>(ctx)->process_status(*pkt);
							},

					// use default get_host_time()
					NULL, &processor);

	inno_log_verify(ret == 0, "set_callbacks failed %d", ret);

	ret = inno_lidar_start(handle);
	inno_log_verify(ret == 0, "start failed %d", ret);

	std::thread act(ThreadAct, std::ref(InnoLidarObject));
	act.join();

	while (!processor.is_done())
	{
		std::this_thread::sleep_for(std::chrono::milliseconds(10));
	}

	mainLog.LogInfo() << "Done.";
	execClient.ReportExecutionState(ara::exec::ExecutionState::kTerminating);

	ret = inno_lidar_stop(handle);
	inno_log_verify(ret == 0, "stop failed %d", ret);

	ret = inno_lidar_close(handle);
	inno_log_verify(ret == 0, "close failed %d", ret);

	return 0;
}
